﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using RetailForce.Fiscalisation;
using RetailForce.Fiscalisation.Configuration;
using RetailForce.Fiscalisation.Implementation.Germany;
using RetailForce.Fiscalisation.Model.Document;
using RetailForce.Fiscalisation.ServiceClient;

namespace RetailForce.Fiscalisation.Samples.FiscalServiceClientSample
{
    public partial class SampleForm : Form
    {
        private FiscalServiceClient _serviceClient = new FiscalServiceClient("http://localhost:7678/api/v1/");
        private FiscalClient _fiscalClient;

        public SampleForm()
        {
            InitializeComponent();
            FiscalClient[] clients = _serviceClient.GetClients();

            cmbClient.Items.Add(new FiscalClient() { StoreName = "Empty" });
            cmbClient.Items.AddRange(clients);
            
        }

        public void ValidateDocument()
        {
            Document document = GetDocument();

            document.FiscalDocumentNumber = 2;
            document.FiscalDocumentRevision = 1;
            List<ValidationError> errors = _serviceClient.ValidateDocument(document);

            lstValidationError.Items.Clear();
            if (errors != null && errors.Count > 0)
            {
                lstValidationError.Items.AddRange(errors.ToArray());
            }
            else MessageBox.Show("Validation successful.");
        }

        public FiscalResponse StoreDocument()
        {
            Document document = GetDocument();

            FiscalResponse fiscalResponse = _serviceClient.CreateDocument(_fiscalClient.UniqueClientId, document.DocumentType);

            document.FiscalDocumentNumber = fiscalResponse.FiscalisationDocumentNumber;
            document.FiscalDocumentRevision = fiscalResponse.FiscalisationDocumentRevision;

            return _serviceClient.StoreDocument(document);
        }

        /// <summary>
        /// Creates the document from the given parameters.
        /// </summary>
        /// <returns></returns>
        public Document GetDocument()
        {
            Document document = new Document()
            {
                CreateDate = DateTime.Now,
                DocumentGuid = Guid.NewGuid(),
                BookDate = DateTime.Now,
                IsTraining = false,
                UniqueClientId = _fiscalClient.UniqueClientId,
                User = new RetailForce.Fiscalisation.Model.User()
                {
                    Caption = "Testuser",
                    Id = "1234"
                }
            };

            document.Positions = new List<DocumentPositionBase>();

            foreach (Model.SimpleItem item in lstPositions.Items)
                document.Positions.Add(GetItemPosition(item));

            int x = 0;
            foreach (DocumentPositionItem itemPos in document.Positions)
            {
                itemPos.PositionNumber = x;
                x++;
            }

            document.Payments = new List<DocumentPayment>();
            document.Payments.Add(new DocumentPayment() { Amount = document.Positions.Cast<DocumentPositionItem>().Sum(s => s.GrossValue), CurrencyIsoCode = "EUR" });

            return document;
        }

        public DocumentPositionItem GetItemPosition(Model.SimpleItem item)
        {
            DocumentPositionItem itemPos = new DocumentPositionItem()
            {
                ItemId = item.ItemId,
                ItemCaption = item.Caption,
                GrossValue = item.Price,
                NetValue = RetailForce.Fiscalisation.Model.Vat.GetNetValue(item.Vat.VatPercent, item.Price),
                TaxValue = item.Price - RetailForce.Fiscalisation.Model.Vat.GetNetValue(item.Vat.VatPercent, item.Price),
                BaseGrossValue = item.Price,
                VatPercent = item.Vat.VatPercent,
                VatIdentification = item.Vat.VatId,
                Quantity = 1
            };

            return itemPos;
        }


        private void SampleForm_Load(object sender, EventArgs e)
        {
            
        }

        private void cmbClient_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cmbClient.SelectedItem != null)
            {
                _fiscalClient = (FiscalClient)cmbClient.SelectedItem;
                bool enabled = false;
                if (_fiscalClient.StoreName != "Empty")
                    enabled = true;

                cmbItem.Enabled = enabled;
                lstPositions.Enabled = enabled;
                btnAddPosition.Enabled = enabled;
                btnDeletePosition.Enabled = enabled;
                btnStore.Enabled = enabled;
                btnValidate.Enabled = enabled;
                lstValidationError.Enabled = enabled;

                cmbItem.Items.Clear();
                cmbItem.Items.AddRange(_GetItems());
            }
        }

        private Model.SimpleItem[] _GetItems()
        {
            if (_fiscalClient == null || _fiscalClient.StoreName == "Empty") return null;

            Model.SimpleVat countryVat;
            switch(_fiscalClient.FiscalCountry)
            {
                case FiscalCountry.Germany: countryVat = new Model.SimpleVat() { }; break;
                default: return null;
            }

            List<Model.SimpleItem> simpleItems = new List<Model.SimpleItem>();
            simpleItems.Add(new Model.SimpleItem() { ItemId = "000452", Caption = "Blume blau", Price = 4.50m, Vat = _GetVat() });
            simpleItems.Add(new Model.SimpleItem() { ItemId = "000714", Caption = "Blume grün", Price = 4.30m, Vat = _GetVat() });
            simpleItems.Add(new Model.SimpleItem() { ItemId = "000322", Caption = "Rose", Price = 6.70m, Vat = _GetVat() });
            simpleItems.Add(new Model.SimpleItem() { ItemId = "000169", Caption = "Tulpe", Price = 2.30m, Vat = _GetVat() });
            simpleItems.Add(new Model.SimpleItem() { ItemId = "001180", Caption = "Hochzeitsstrauss", Price = 22.80m, Vat = _GetVat() });

            return simpleItems.ToArray();
        }

        private Model.SimpleVat _GetVat()
        {
            if (_fiscalClient == null || _fiscalClient.StoreName == "Empty") return null;

            switch (_fiscalClient.FiscalCountry)
            {
                case FiscalCountry.Germany: return new Model.SimpleVat() { VatId = 1, VatPercent = 19 };
                default: return null;
            }
        }

        private void btnAddPosition_Click(object sender, EventArgs e)
        {
            if (cmbItem.SelectedItem != null)
                lstPositions.Items.Add(cmbItem.SelectedItem);
        }

        private void btnDeletePosition_Click(object sender, EventArgs e)
        {
            if (lstPositions.SelectedIndex > 0)
                lstPositions.Items.RemoveAt(lstPositions.SelectedIndex);
        }

        private void btnValidate_Click(object sender, EventArgs e)
        {
            ValidateDocument();
        }

        private void btnStore_Click(object sender, EventArgs e)
        {
            FiscalResponse fiscalResponse = StoreDocument();

            if (fiscalResponse == null) return;

            if (fiscalResponse.FiscalCountry == FiscalCountry.Germany)
            {
                lstValidationError.Items.Clear();
                lstValidationError.Items.Add($"ProcessType: {fiscalResponse.ProcessType()}");
                lstValidationError.Items.Add($"ProcessData: {fiscalResponse.ProcessData()}");
                lstValidationError.Items.Add($"StartTime: {fiscalResponse.TransactionStartTime()}");
                lstValidationError.Items.Add($"EndTime: {fiscalResponse.TransactionEndTime()}");
                lstValidationError.Items.Add($"DocumentNumber: {fiscalResponse.FiscalisationDocumentNumber}");
                lstValidationError.Items.Add($"DocumentRevision: {fiscalResponse.FiscalisationDocumentRevision}");
                lstValidationError.Items.Add($"QrCodeData: {fiscalResponse.QrCodeDataString()}");
                lstValidationError.Items.Add($"Signature: {fiscalResponse.Signature}");

            }
        }
    }
}
